@@ -103,7 +103,9 @@ SignEncryptFilesCommand::Private::Private(SignEncryptFilesCommand *qq, KeyListCo
       shared_qq(qq, kdtools::nodelete()),
       controller()
 {
-    controller.setOperationMode(SignEncryptFilesController::SignAllowed | SignEncryptFilesController::EncryptAllowed | SignEncryptFilesController::ArchiveAllowed);
+    controller.setOperationMode(SignEncryptFilesController::SignSelected |
+                                SignEncryptFilesController::EncryptSelected |
+                                SignEncryptFilesController::ArchiveAllowed);
 }
 
 SignEncryptFilesCommand::Private::~Private()
@@ -167,7 +169,7 @@ void SignEncryptFilesCommand::setSigningPolicy(Policy policy)
         mode |= SignEncryptFilesController::SignDisallowed;
         break;
     case Force:
-        mode |= SignEncryptFilesController::SignForced;
+        mode |= SignEncryptFilesController::SignSelected;
         break;
     }
     try {
@@ -184,7 +186,7 @@ Policy SignEncryptFilesCommand::signingPolicy() const
         return NoPolicy;
     case SignEncryptFilesController::SignAllowed:
         return Allow;
-    case SignEncryptFilesController::SignForced:
+    case SignEncryptFilesController::SignSelected:
         return Force;
     case SignEncryptFilesController::SignDisallowed:
         return Deny;
@@ -204,7 +206,7 @@ void SignEncryptFilesCommand::setEncryptionPolicy(Policy policy)
         mode |= SignEncryptFilesController::EncryptDisallowed;
         break;
     case Force:
-        mode |= SignEncryptFilesController::EncryptForced;
+        mode |= SignEncryptFilesController::EncryptSelected;
         break;
     }
     try {
@@ -221,13 +223,48 @@ Policy SignEncryptFilesCommand::encryptionPolicy() const
         return NoPolicy;
     case SignEncryptFilesController::EncryptAllowed:
         return Allow;
-    case SignEncryptFilesController::EncryptForced:
+    case SignEncryptFilesController::EncryptSelected:
         return Force;
     case SignEncryptFilesController::EncryptDisallowed:
         return Deny;
     }
 }
 
+void SignEncryptFilesCommand::setArchivePolicy(Policy policy)
+{
+    unsigned int mode = d->controller.operationMode();
+    mode &= ~SignEncryptFilesController::ArchiveMask;
+    switch (policy) {
+    case NoPolicy:
+    case Allow:
+        mode |= SignEncryptFilesController::ArchiveAllowed;
+        break;
+    case Deny:
+        mode |= SignEncryptFilesController::ArchiveDisallowed;
+        break;
+    case Force:
+        mode |= SignEncryptFilesController::ArchiveForced;
+        break;
+    }
+    d->controller.setOperationMode(mode);
+}
+
+Policy SignEncryptFilesCommand::archivePolicy() const
+{
+    const unsigned int mode = d->controller.operationMode();
+    switch (mode & SignEncryptFilesController::ArchiveMask) {
+    case SignEncryptFilesController::ArchiveAllowed:
+        return Allow;
+    case SignEncryptFilesController::ArchiveForced:
+        return Force;
+    case SignEncryptFilesController::ArchiveDisallowed:
+        return Deny;
+    default:
+        assert(!"This should not happen!");
+        return NoPolicy;
+    }
+}
+
 void SignEncryptFilesCommand::setProtocol(GpgME::Protocol proto)
 {
     d->controller.setProtocol(proto);
@@ -244,7 +281,7 @@ void SignEncryptFilesCommand::doStart()
     try {
 
         if (d->files.empty()) {
-            d->files = d->selectFiles();
+            d->files = selectFiles();
         }
         if (d->files.empty()) {
             d->finished();
@@ -268,9 +305,9 @@ void SignEncryptFilesCommand::doCancel()
     d->controller.cancel();
 }
 
-QStringList SignEncryptFilesCommand::Private::selectFiles() const
+QStringList SignEncryptFilesCommand::selectFiles() const
 {
-    return FileDialog::getOpenFileNames(parentWidgetOrView(), i18n("Select One or More Files to Sign and/or Encrypt"), QStringLiteral("enc"));
+    return FileDialog::getOpenFileNames(d->parentWidgetOrView(), i18n("Select One or More Files to Sign and/or Encrypt"), QStringLiteral("enc"));
 }
 
 #undef d
