@@ -176,37 +176,27 @@ DecryptVerifyFilesController::Private::Private(DecryptVerifyFilesController *qq)
 
 void DecryptVerifyFilesController::Private::slotWizardOperationPrepared()
 {
-    try {
-        ensureWizardCreated();
-        std::vector<shared_ptr<Task> > tasks = buildTasks(m_filesAfterPreparation, shared_ptr<OverwritePolicy>(new OverwritePolicy(m_wizard)));
-        kleo_assert(m_runnableTasks.empty());
-        m_runnableTasks.swap(tasks);
-
-        shared_ptr<TaskCollection> coll(new TaskCollection);
-        Q_FOREACH (const shared_ptr<Task> &i, m_runnableTasks) {
-            q->connectTask(i);
-        }
-        coll->setTasks(m_runnableTasks);
-        m_wizard->setTaskCollection(coll);
-
-        QTimer::singleShot(0, q, SLOT(schedule()));
-
-    } catch (const Kleo::Exception &e) {
-        reportError(e.error().encodedError(), e.message());
-    } catch (const std::exception &e) {
-        reportError(gpg_error(GPG_ERR_UNEXPECTED),
-                    i18n("Caught unexpected exception in DecryptVerifyFilesController::Private::slotWizardOperationPrepared: %1",
-                         QString::fromLocal8Bit(e.what())));
-    } catch (...) {
-        reportError(gpg_error(GPG_ERR_UNEXPECTED),
-                    i18n("Caught unknown exception in DecryptVerifyFilesController::Private::slotWizardOperationPrepared"));
+    ensureWizardCreated();
+    std::vector<shared_ptr<Task> > tasks = buildTasks(m_filesAfterPreparation, shared_ptr<OverwritePolicy>(new OverwritePolicy(m_wizard)));
+    if (tasks.empty()) {
+        reportError(makeGnuPGError(GPG_ERR_ASS_NO_INPUT), i18n("No usable inputs found"));
     }
+    kleo_assert(m_runnableTasks.empty());
+    m_runnableTasks.swap(tasks);
+
+    shared_ptr<TaskCollection> coll(new TaskCollection);
+    Q_FOREACH (const shared_ptr<Task> &i, m_runnableTasks) {
+        q->connectTask(i);
+    }
+    coll->setTasks(m_runnableTasks);
+    m_wizard->setTaskCollection(coll);
+
+    QTimer::singleShot(0, q, SLOT(schedule()));
 }
 
 void DecryptVerifyFilesController::Private::slotWizardCanceled()
 {
     qCDebug(KLEOPATRA_LOG);
-    reportError(gpg_error(GPG_ERR_CANCELED), i18n("User canceled"));
 }
 
 void DecryptVerifyFilesController::doTaskDone(const Task *task, const shared_ptr<const Task::Result> &result)
@@ -284,7 +274,7 @@ struct FindExtension : std::unary_function<shared_ptr<ArchiveDefinition>, bool>
 };
 }
 
-shared_ptr<ArchiveDefinition> pick_archive_definition(GpgME::Protocol proto, const std::vector< shared_ptr<ArchiveDefinition> > &ads, const QString &filename)
+shared_ptr<ArchiveDefinition> DecryptVerifyFilesController::pick_archive_definition(GpgME::Protocol proto, const std::vector< shared_ptr<ArchiveDefinition> > &ads, const QString &filename)
 {
     const QFileInfo fi(outputFileName(filename));
     QString extension = fi.completeSuffix();
@@ -344,7 +334,7 @@ QStringList DecryptVerifyFilesController::Private::prepareWizardFromPassedFiles(
             }
             // ### end FIXME
             else if (mayBeOpaqueSignature(classification) || mayBeCipherText(classification)) {
-                op->setMode(DecryptVerifyOperationWidget::DecryptVerifyOpaque, pick_archive_definition(proto, archiveDefinitions, fname));
+                op->setMode(DecryptVerifyOperationWidget::DecryptVerifyOpaque, q->pick_archive_definition(proto, archiveDefinitions, fname));
             } else {
                 op->setMode(DecryptVerifyOperationWidget::VerifyDetachedWithSignature);
             }
@@ -367,7 +357,7 @@ QStringList DecryptVerifyFilesController::Private::prepareWizardFromPassedFiles(
                 DecryptVerifyOperationWidget *const op = m_wizard->operationWidget(counter++);
                 kleo_assert(op != 0);
                 op->setArchiveDefinitions(archiveDefinitions);
-                op->setMode(DecryptVerifyOperationWidget::DecryptVerifyOpaque, pick_archive_definition(proto, archiveDefinitions, fname));
+                op->setMode(DecryptVerifyOperationWidget::DecryptVerifyOpaque, q->pick_archive_definition(proto, archiveDefinitions, fname));
                 op->setInputFileName(fname);
                 fileNames.push_back(fname);
             } else {
@@ -390,10 +380,6 @@ QStringList DecryptVerifyFilesController::Private::prepareWizardFromPassedFiles(
 
     kleo_assert(counter == static_cast<unsigned>(fileNames.size()));
 
-    if (!counter) {
-        throw Kleo::Exception(makeGnuPGError(GPG_ERR_ASS_NO_INPUT), i18n("No usable inputs found"));
-    }
-
     m_wizard->setOutputDirectory(heuristicBaseDirectory(m_passedFiles));
     return fileNames;
 }
